/* global Util, UtilDevTools, diff */
function DTCustomizationService(templateManager, ZHPromise, DTServerService, DTMessages, DTRenderService){
    var self = this;
    var zeedhiTypes = UtilDevTools.getZeedhiTypes();
    var constProperties = UtilDevTools.getIgnoredZeedhiTypes().concat(['__indexPath', 'fileName']);
    var originalItems = {};
    var customContainerItems = {};
    var newItems = {};
    var NEW = "NEW";

    var resetCustomItems = function(){
        customContainerItems = {};
        newItems = {};
    };

    var sendConfirmMessage = function(message){
        var defer = ZHPromise.defer();
        UtilDevTools.getMainWindow().showConfirmMessageInMainWindow(message, defer.resolve, defer.reject);
        return defer.promise;
    };

    var updateOriginalItemsPath = function(item, pathToSave){
        if(!isNewCustomId(item.id)){
            originalItems[item.id].item._path = pathToSave;
        }
    };

    var saveCustom = function(item, containerId, pathToSave){
        DTServerService.saveCustomComponent(item.id, containerId, pathToSave, item).then(function(response){
            if(response.success){
                self.renderCustomizationInMainWindow();
                updateOriginalItemsPath(item, pathToSave);
            }
        });
    };

    var isNewCustomId = function(customId){
        return customId.toString().indexOf(NEW) > -1 ;
    };

    var updateCustomItems = function(containerId){
        return DTServerService.getContainer(containerId).then(function(data){
            customContainerItems = data.updatedItems;
            newItems = data.newItems;
        }).catch(function (err){
            customContainerItems = {};
            newItems = {};
            console.log(err);
        });
    };

    var applyObjectCustomization = function(obj){
        var customObj = Util.clone(customContainerItems[obj.id]);
        if(customObj){
            delete customObj._path;
            Object.keys(obj).forEach(function(key){
                if(customObj.hasOwnProperty(key)){
                    obj[key] = customObj[key];
                    delete customObj[key];
                } else if(zeedhiTypes.indexOf(key) == -1 && constProperties.indexOf(key) == -1){
                    delete obj[key];
                }
            });
            Object.keys(customObj).forEach(function(key){
                obj[key] = customObj[key];
            });
        }
        return obj;
    };

    var buildCustomizationItems = function(object, type, items){
        var clonedObj = Util.clone(object);
        items = items || {};
        if(object.hasOwnProperty("id")){
            zeedhiTypes.forEach(function(key){
                if(key != '__metadata' && object[key]){
                    object[key].forEach(function(item){
                        items = buildCustomizationItems(item, key, items);
                    });
                }
                delete clonedObj[key];
            });
            var custom = Util.clone(customContainerItems[object.id]);
            clonedObj._path = custom ? custom._path : "";
            items[object.id] = {
                item: clonedObj,
                type: type || "container"
            };
            object = applyObjectCustomization(object);
        }
        return items;
    };

    var hasBeenAppliedInContainer = function(zeedhiArray, item){
        return zeedhiArray.some(function(oldItem, index){
            if(oldItem.id == item.id){
                zeedhiArray[index] = applyObjectCustomization(item);
                return true;
            }
            return false;
        });
    };

    var pushItemInArray = function(zeedhiArray, item, index){
        var hasUpdated = hasBeenAppliedInContainer(zeedhiArray, item);

        if(!hasUpdated){
            if(zeedhiArray.length <= index+1){
                zeedhiArray.push(item);
            } else {
                zeedhiArray.splice(index+1, 0, item);
            }
        }
    };

    var applyItemInContainer = function(item, container){
        var path = item._path;
        delete item._path;
        path.reduce(function(current, key, index) {
            if(index === path.length - 1){
                pushItemInArray(current, item, key);
            }
            return current[key];
        }, container);
    };

    var removeItemInContainer = function(item, container){
        var path = item._path;
        delete item._path;
        path.reduce(function(current, key, index) {
            if(index === path.length - 1){
                current.splice(key, 1);
            }
            return current[key];
        }, container);
    };

    var applyNewCustomItems = function(container){
        Object.keys(newItems).forEach(function(item, index){
            applyItemInContainer(Util.clone(newItems[item]), container);
        });
    };

    var applyContainerCustomization = function(container){
        return updateCustomItems(container.id).then(function(){
            var items = buildCustomizationItems(container);
            applyNewCustomItems(container);
            return {"container": container, "items": items};
        });
    };

    var isSetted = false;
    var accessEditMode = false;

    this.setAccessEditMode = function(hasAccess){
        if(!isSetted){
            accessEditMode = hasAccess;
            isSetted = true;
        }
    };

    this.getAccessEditMode = function(){
        return !templateManager.inCustomizationMode || accessEditMode;
    };

    this.buildContainerCustomization = function(container){
        return applyContainerCustomization(container).then(function(customization){
            originalItems = customization.items;
            return customization.container;
        });
    };

    this.renderCustomizationInMainWindow = UtilDevTools.buildMainWindowMethod(function(container){
        container = container || templateManager.container;
        var metadata = Util.clone(container.__metadata);
        applyContainerCustomization(metadata).then(function(customization){
            DTRenderService.renderContainer(customization.container);
        });
    });

    this.deleteItemCustomization = function(itemId){
        sendConfirmMessage(DTMessages.CONFIRM_DELETE)
            .then(function(){
                return DTServerService.deleteItemCustomization(itemId);
            })
            .then(function(wasDeleted){
                if(wasDeleted){
                    var container = Util.clone(templateManager.container.__metadata);
                    var item;
                    if(isNewCustomId(itemId)){
                        item = Util.clone(newItems)[itemId];
                        delete newItems[itemId];
                        removeItemInContainer(item, container);
                    } else {
                        item = Util.clone(originalItems)[itemId].item;
                        delete customContainerItems[itemId];
                        applyItemInContainer(item, container);
                    }
                    DTRenderService.renderContainer(container);
                }
            });
    };

    this.deleteCustomFile = function(fileName){
        sendConfirmMessage(DTMessages.CONFIRM_DELETE).then(function(){
            DTServerService.deleteCustomFile(fileName);
        });
    };

    this.resetContainer = function(){
        var queryToReset = templateManager.container.id;
        sendConfirmMessage(DTMessages.CONFIRM_RESET)
            .then(function(){
                return DTServerService.resetContainer(queryToReset);
            })
            .then(function(wasDeleted){
                if(wasDeleted){
                    resetCustomItems();
                    DTRenderService.renderContainer(templateManager.container.__original);
                }
            });
    };

    this.saveItemCustomization = function(item, containerId, pathToSave){
        var itemId = item.id;
        DTServerService.getCustomComponent(itemId, containerId).then(function(newCustomComponent){
            var custom = isNewCustomId(itemId) ? newItems[itemId] : customContainerItems[itemId];
            var oldCustom = custom || {};
            var needUpdate = oldCustom ^ newCustomComponent;
            var diffs = diff.getDiff(oldCustom, newCustomComponent);

            if(needUpdate || Object.keys(diffs).length){
                sendConfirmMessage(DTMessages.CUSTOM_OUTDATED).then(function(){
                    saveCustom(item, containerId, pathToSave);
                }).catch(function(){
                    sendConfirmMessage(DTMessages.RELOAD_CONTAINER).then(function(){
                        self.renderCustomizationInMainWindow();
                    });
                });
            } else {
                saveCustom(item, containerId, pathToSave);
            }
        });
    };
}
